# Update class names from the immediately prior version only
# to ensure that cycles are not possible

from apptools.persistence.updater import Updater


def update_project(self, state):
    print 'updating to v3'
    metadata = state['metadata']
    metadata['version'] = 3
    metadata['finished'] = True
    return state

class Update3(Updater):

    def __init__(self):

        self.refactorings = {
            ("__main__", "Foo1"): ("__main__", "Foo2"),
        }

        self.setstates = {
            ("cplab.project", "Project"):  update_project
        }



"""The workbench window for the permissions framework example."""


# Enthought library imports.
from pyface.action.api import Action, Group, MenuManager
from pyface.workbench.api import EditorManager, WorkbenchWindow
from pyface.workbench.api import Perspective, PerspectiveItem
from pyface.workbench.action.api import MenuBarManager
from pyface.workbench.action.api import ToolBarManager
from pyface.workbench.action.api import ViewMenuManager
from apptools.permissions.api import SecureProxy
from apptools.permissions.action.api import UserMenuManager
from traits.api import Callable, HasTraits, List, Instance

# Local imports.
from permissions import NewPersonPerm
from person import Person
from toolkit_editor import ToolkitEditor


class ExampleEditorManager(EditorManager):
    """An editor manager that supports the editor memento protocol."""

    #######################################################################
    # 'IEditorManager' interface.
    #######################################################################

    def create_editor(self, window, obj, kind):
        """Reimplemented to create an editor appropriate to the object being
        edited.
        """

        if isinstance(obj, HasTraits):
            # The superclass handles Traits objects.
            editor = super(ExampleEditorManager, self).create_editor(window, obj, kind)
        else:
            # Assume it is handled by a toolkit specific editor.
            editor = ToolkitEditor(window=window, obj=obj)

        return editor


class ExampleWorkbenchWindow(WorkbenchWindow):
    """A simple example of using the workbench window."""

    #### 'WorkbenchWindow' interface ##########################################

    # The available perspectives.
    perspectives = [
        Perspective(
            name     = 'Foo',
            contents = [
                PerspectiveItem(id='Black', position='bottom'),
                PerspectiveItem(id='Debug', position='left')
            ]
        ),

        Perspective(
            name     = 'Bar',
            contents = [
                PerspectiveItem(id='Debug', position='left')
            ]
        )
    ]

    #### Private interface ####################################################

    # The Exit action.
    _exit_action = Instance(Action)

    # The New Person action.
    _new_person_action = Instance(Action)

    ###########################################################################
    # 'ApplicationWindow' interface.
    ###########################################################################

    def _editor_manager_default(self):
        """ Trait initializer.

        Here we return the replacement editor manager.
        """

        return ExampleEditorManager()

    def _menu_bar_manager_default(self):
        """Trait initializer."""

        file_menu = MenuManager(self._new_person_action, self._exit_action,
                name='&File', id='FileMenu')
        view_menu = ViewMenuManager(name='&View', id='ViewMenu', window=self)
        user_menu = UserMenuManager(id='UserMenu', window=self)

        return MenuBarManager(file_menu, view_menu, user_menu, window=self)

    def _tool_bar_manager_default(self):
        """Trait initializer."""

        return ToolBarManager(self._exit_action, show_tool_names=False)

    ###########################################################################
    # 'WorkbenchWindow' interface.
    ###########################################################################

    def _views_default(self):
        """Trait initializer."""

        from secured_debug_view import SecuredDebugView

        return [SecuredDebugView(window=self)]

    ###########################################################################
    # Private interface.
    ###########################################################################

    def __exit_action_default(self):
        """Trait initializer."""

        return Action(name='E&xit', on_perform=self.workbench.exit)

    def __new_person_action_default(self):
        """Trait initializer."""

        # Create the action and secure it with the appropriate permission.
        act = Action(name='New Person', on_perform=self._new_person)
        act = SecureProxy(act, permissions=[NewPersonPerm])

        return act

    def _new_person(self):
        """Create a new person."""

        self.workbench.edit(Person(name='New', age=100))

#### EOF ######################################################################



#------------------------------------------------------------------------------
# Copyright (c) 2008, Riverbank Computing Limited
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Riverbank Computing Limited
# Description: <Enthought permissions package component>
#------------------------------------------------------------------------------


# Standard library imports.
import errno
import socket

# Enthought library imports.
from apptools.permissions.default.api import IUserStorage, UserStorageError
from traits.api import HasTraits, provides, List, Str

# Local imports.
from proxy_server import ProxyServer


@provides(IUserStorage)
class UserStorage(HasTraits):
    """This implements a user database accessed via XML RPC."""



    #### 'IUserStorage' interface #############################################

    capabilities = List(Str)

    ###########################################################################
    # 'IUserStorage' interface.
    ###########################################################################

    def add_user(self, name, description, password):
        """Add a new user."""

        try:
            ProxyServer.add_user(name, description, password, ProxyServer.key)
        except Exception, e:
            raise UserStorageError(ProxyServer.error(e))

    def authenticate_user(self, name, password):
        """Return the tuple of the user name, description, and blob if the user
        was successfully authenticated."""

        try:
            key, name, description, blob = ProxyServer.authenticate_user(name,
                    password)

            # We don't save the cache because we should be about to read the
            # real permission ids and we do it then.
            ProxyServer.cache = description, blob, []
        except Exception, e:
            # See if we couldn't connect to the server.
            if not isinstance(e, socket.error):
                raise UserStorageError(ProxyServer.error(e))

            err, _ = e.args

            if err != errno.ECONNREFUSED:
                raise UserStorageError(ProxyServer.error(e))

            try:
                ok = ProxyServer.read_cache()
            except Exception, e:
                raise UserStorageError(str(e))

            if not ok:
                raise UserStorageError(ProxyServer.error(e))

            # We are in "disconnect" mode.
            key = None
            description, blob, _ = ProxyServer.cache

        ProxyServer.key = key

        return name, description, blob

    def delete_user(self, name):
        """Delete a new user."""

        try:
            ProxyServer.delete_user(name, ProxyServer.key)
        except Exception, e:
            raise UserStorageError(ProxyServer.error(e))

    def is_empty(self):
        """See if the database is empty."""

        # We leave it to the policy storage to answer this question.
        return False

    def matching_users(self, name):
        """Return the full name and description of all the users that match the
        given name."""

        try:
            return ProxyServer.matching_users(name, ProxyServer.key)
        except Exception, e:
            raise UserStorageError(ProxyServer.error(e))

    def modify_user(self, name, description, password):
        """Update the description and password for the given user."""

        try:
            ProxyServer.modify_user(name, description, password,
                    ProxyServer.key)
        except Exception, e:
            raise UserStorageError(ProxyServer.error(e))

    def unauthenticate_user(self, user):
        """Unauthenticate the given user."""

        if ProxyServer.key is None:
            ok = True
        else:
            try:
                ok = ProxyServer.unauthenticate_user(ProxyServer.key)
            except Exception, e:
                raise UserStorageError(ProxyServer.error(e))

        if ok:
            ProxyServer.key = ''
            ProxyServer.cache = None

        return ok

    def update_blob(self, name, blob):
        """Update the blob for the given user."""

        # Update the cache.
        description, _, perm_ids = ProxyServer.cache
        ProxyServer.cache = description, blob, perm_ids

        if ProxyServer.key is None:
            # Write the cache and tell the user about any errors.
            ProxyServer.write_cache()
        else:
            try:
                ProxyServer.update_blob(name, blob, ProxyServer.key)
            except Exception, e:
                raise UserStorageError(ProxyServer.error(e))

            # Write the cache but ignore any errors.
            try:
                ProxyServer.write_cache()
            except:
                pass

    def update_password(self, name, password):
        """Update the password for the given user."""

        # If the remote server disappeared after the capabilities were read but
        # before the user was authenticated then we could get here.
        if ProxyServer.key is None:
            raise UserStorageError("It is not possible to change password "
                    "when disconnected from the permissions server.")

        try:
            ProxyServer.update_password(name, password, ProxyServer.key)
        except Exception, e:
            raise UserStorageError(ProxyServer.error(e))

    ###########################################################################
    # Trait handlers.
    ###########################################################################

    def _capabilities_default(self):
        """Return the storage capabilities."""

        try:
            caps = ProxyServer.capabilities()
        except:
            caps = []

        return caps



#------------------------------------------------------------------------------
# Copyright (c) 2008, Riverbank Computing Limited
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Riverbank Computing Limited
# Description: <Enthought permissions package component>
#------------------------------------------------------------------------------


# Enthought library imports.
from apptools.permissions.default.api import IPolicyStorage, \
        PolicyStorageError
from traits.api import HasTraits, provides

# Local imports.
from proxy_server import ProxyServer


@provides(IPolicyStorage)
class PolicyStorage(HasTraits):
    """This implements a policy database accessed via XML RPC."""



    ###########################################################################
    # 'IPolicyStorage' interface.
    ###########################################################################

    def add_role(self, name, description, perm_ids):
        """Add a new role."""

        try:
            ProxyServer.add_role(name, description, perm_ids, ProxyServer.key)
        except Exception, e:
            raise PolicyStorageError(ProxyServer.error(e))

    def all_roles(self):
        """Return a list of all roles."""

        try:
            return ProxyServer.all_roles(ProxyServer.key)
        except Exception, e:
            raise PolicyStorageError(ProxyServer.error(e))

    def delete_role(self, name):
        """Delete a role."""

        try:
            ProxyServer.delete_role(name, ProxyServer.key)
        except Exception, e:
            raise PolicyStorageError(ProxyServer.error(e))

    def get_assignment(self, user_name):
        """Return the details of the assignment for the given user name."""

        try:
            return ProxyServer.get_assignment(user_name, ProxyServer.key)
        except Exception, e:
            raise PolicyStorageError(ProxyServer.error(e))

    def get_policy(self, name):
        """Return the details of the policy for the given user name."""

        description, blob, perm_ids = ProxyServer.cache

        if ProxyServer.key is not None:
            try:
                name, perm_ids = ProxyServer.get_policy(name, ProxyServer.key)
            except Exception, e:
                raise PolicyStorageError(ProxyServer.error(e))

            # Save the permissions ids in the persistent cache.
            ProxyServer.cache = description, blob, perm_ids

            try:
                ProxyServer.write_cache()
            except:
                pass

        return name, perm_ids

    def is_empty(self):
        """See if the database is empty."""

        try:
            return ProxyServer.is_empty_policy()
        except Exception, e:
            raise PolicyStorageError(ProxyServer.error(e))

    def matching_roles(self, name):
        """Return the full name, description and permissions of all the roles
        that match the given name."""

        try:
            return ProxyServer.matching_roles(name, ProxyServer.key)
        except Exception, e:
            raise PolicyStorageError(ProxyServer.error(e))

    def modify_role(self, name, description, perm_ids):
        """Update the description and permissions for the given role."""

        try:
            ProxyServer.modify_role(name, description, perm_ids,
                    ProxyServer.key)
        except Exception, e:
            raise PolicyStorageError(ProxyServer.error(e))

    def set_assignment(self, user_name, role_names):
        """Save the roles assigned to a user."""

        try:
            ProxyServer.set_assignment(user_name, role_names, ProxyServer.key)
        except Exception, e:
            raise PolicyStorageError(ProxyServer.error(e))



from email.mime.multipart import MIMEMultipart
import unittest

try:
    # On Python 3, mock is part of the standard library,
    from unittest import mock
except ImportError:
    # Whereas on Python 2 it is not.
    import mock

from apptools.logger.plugin.logger_service import LoggerService


class LoggerServiceTestCase(unittest.TestCase):
    def test_create_email_message(self):
        logger_service = LoggerService()
        with mock.patch.object(logger_service, 'whole_log_text')\
                as mocked_log_txt:
            mocked_log_txt.return_value = "Dummy log data"
            msg = logger_service.create_email_message(
                fromaddr='', toaddrs='', ccaddrs='', subject='', priority=''
            )
        self.assertIsInstance(msg, MIMEMultipart)

    def test_create_email_message_with_user_data(self):
        # We used a mocked logger service which doesn't depend on the
        # application trait and the presence of extensions to the extension
        # point `apptools.logger.plugin.mail_files`
        class MockedLoggerService(LoggerService):
            def _get_mail_files(self):
                return [lambda zip_file : None]

        logger_service = MockedLoggerService()
        with mock.patch.object(logger_service, 'whole_log_text')\
                as mocked_log_txt:
            mocked_log_txt.return_value = "Dummy log data"
            msg = logger_service.create_email_message(
                fromaddr='', toaddrs='', ccaddrs='', subject='', priority='',
                include_userdata=True,
            )
        self.assertIsInstance(msg, MIMEMultipart)



#!/usr/bin/env python

#------------------------------------------------------------------------------
# Copyright (c) 2008, Riverbank Computing Limited
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Riverbank Computing Limited
# Description: <Enthought permissions package component>
#------------------------------------------------------------------------------


# Standard library imports.
import errno
import optparse
import socket
import sys
import xmlrpclib


# The default IP address to connect to.
DEFAULT_ADDR = socket.gethostbyname(socket.gethostname())

# The default port to connect to.
DEFAULT_PORT = 3800


def check_blob(opts):
    """Check the blob was specified with the --blob flag and return the value.
    """

    if not opts.blob:
        raise Exception("the --blob flag to specify a blob")

    return opts.blob


def check_name(opts):
    """Check the name was specified with the --name flag and return the value.
    """

    if not opts.name:
        raise Exception("the --name flag to specify a name")

    return opts.name


def check_password(opts):
    """Check the password was specified with the --password flag and return the
    value."""

    if not opts.password:
        raise Exception("the --password flag to specify a password")

    return opts.password


def add_role(opts):
    """Add a new role."""

    return [check_name(opts), opts.description, opts.permissions, opts.key]


def add_user(opts):
    """Add a new user."""

    return [check_name(opts), opts.description, check_password(opts), opts.key]


def all_roles(opts):
    """Return all roles."""

    return [opts.key]


def authenticate_user(opts):
    """Authenticate a user and return their details."""

    return [check_name(opts), check_password(opts)]


def delete_role(opts):
    """Delete a role."""

    return [check_name(opts), opts.key]


def delete_user(opts):
    """Delete a user."""

    return [check_name(opts), opts.key]


def capabilities(opts):
    """Get the capabilities of the server."""

    return []


def matching_roles(opts):
    """Get the roles that match a name."""

    return [opts.name, opts.key]


def matching_users(opts):
    """Get the users that match a name."""

    return [opts.name, opts.key]


def get_assignment(opts):
    """Get the details of a user's assignment."""

    return [check_name(opts), opts.key]


def get_policy(opts):
    """Get the details of a user's policy."""

    return [check_name(opts), opts.key]


def is_empty_policy(opts):
    """Check if there is any policy data."""

    return []


def set_assignment(opts):
    """Save the details of a user's assignment."""

    return [check_name(opts), opts.roles, opts.key]


def unauthenticate_user(opts):
    """Unauthenticate a user."""

    return [opts.key]


def modify_role(opts):
    """Modify an existing role."""

    return [check_name(opts), opts.description, opts.permissions, opts.key]


def modify_user(opts):
    """Modify an existing user."""

    return [check_name(opts), opts.description, check_password(opts), opts.key]


def update_blob(opts):
    """Update the blob for a user."""

    return [check_name(opts), check_blob(opts)]


def update_password(opts):
    """Update the password for a user."""

    return [check_name(opts), check_password(opts)]


# The list of actions that can be invoked from the command line.
actions = [
    add_role,
    add_user,
    all_roles,
    authenticate_user,
    capabilities,
    delete_role,
    delete_user,
    get_assignment,
    get_policy,
    is_empty_policy,
    matching_roles,
    matching_users,
    modify_role,
    modify_user,
    set_assignment,
    unauthenticate_user,
    update_blob,
    update_password,
]


def store_list(option, opt_str, value, p):
    """An option parser callback that converts space separated words into a
    list of strings."""

    setattr(p.values, option.dest, value.split(' '))


# Parse the command line.
p = optparse.OptionParser(usage="%prog [options] action", description="This "
        "is a client used to test the XML-RPC server.  The following actions "
        "are supported: %s" % ', '.join([a.func_name for a in actions]))

p.add_option('--ip-address', default=DEFAULT_ADDR, dest='addr',
        help="the IP address to connect to [default: %s]" % DEFAULT_ADDR)
p.add_option('-b', '--blob', dest='blob',
        help="a blob string (used by update_blob)")
p.add_option('-d', '--description', default='', dest='description',
        help="a description (used by add_role, add_user, modify_role, "
                "modify_user)")
p.add_option('-k', '--key', default='', dest='key',
        help="the session key returned by login (used by add_role, add_user, "
                "all_roles, delete_role, delete_user, get_assignment, "
                "get_policy, modify_role, modify_user, set_assignment, "
                "unauthenticate_user)")
p.add_option('-n', '--name', default='', dest='name',
        help="a name (used by add_role, add_user, authenticate_user, "
                "delete_role, delete_user, get_assignment, get_policy, "
                "matching_roles, matching_users, modify_role, modify_user, "
                "set_assignment, update_blob, update_user)")
p.add_option('--permissions', action='callback', callback=store_list,
        default=[], dest='permissions', type='string',
        help="a space separated list of permission names (used by add_role, "
                "modify_role)")
p.add_option('-p', '--password', dest='password',
        help="a password (used by add_user, authenticate_user, modify_user "
                "update_password)")
p.add_option('--port', type='int', default=DEFAULT_PORT, dest='port',
        help="the TCP port to connect to [default: %d]" % DEFAULT_PORT)
p.add_option('--roles', action='callback', callback=store_list, default=[],
        dest='roles', type='string',
        help="a space separated list of role names (used by set_assignment)")
p.add_option('-v', '--verbose', action='store_true', default=False,
        dest='verbose', help="display the progress of the RPC")

opts, args = p.parse_args()

if len(args) != 1:
    p.error("exactly one action must be given")

for action in actions:
    if action.func_name == args[0]:
        break
else:
    p.error("unknown action: %s" % args[0])

# Get the action's arguments.
try:
    action_args = action(opts)
except Exception, e:
    sys.stderr.write("The %s action requires %s\n" % (action.func_name, e))
    sys.exit(2)

# Create the proxy.
proxy = xmlrpclib.ServerProxy(uri='http://%s:%d' % (opts.addr, opts.port),
        verbose=opts.verbose)

try:
    result = getattr(proxy, action.func_name)(*action_args)
except socket.error, e:
    err, msg = e.args

    if err == errno.ECONNREFUSED:
        sys.stderr.write("Unable to connect to permissions server at %s:%d\n" % (opts.addr, opts.port))
    else:
        sys.stderr.write("socket error: %s\n" % msg)

    sys.exit(1)
except xmlrpclib.Fault, e:
    # Extract the text of the exception.  If we don't recognise the format then
    # display the lot.
    tail = e.faultString.find(':')
    if tail < 0:
        msg = e.faultString
    else:
        msg = e.faultString[tail + 1:]

    print "The call raised an exception: %s" % msg
    sys.exit(0)

# Show the result.
print "Result:", result



#------------------------------------------------------------------------------
# Copyright (c) 2007, Riverbank Computing Limited
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Riverbank Computing Limited
# Description: <Enthought undo package component>
#------------------------------------------------------------------------------


# Enthought library imports.
from traits.etsconfig.api import ETSConfig
from pyface.workbench.api import Editor, EditorManager


class _wxLabelEditor(Editor):
    """ _wxLabelEditor is the wx implementation of a label editor. """

    def create_control(self, parent):
        import wx

        w = wx.TextCtrl(parent,style=wx.TE_RICH2)
        style = w.GetDefaultStyle()
        style.SetAlignment(wx.TEXT_ALIGNMENT_CENTER)
        w.SetDefaultStyle(style)

        self._set_text(w)
        self._set_size_and_style(w)

        self.obj.on_trait_change(self._update_text, 'text')
        self.obj.on_trait_change(self._update_size, 'size')
        self.obj.on_trait_change(self._update_style, 'style')

        return w

    def _name_default(self):
        return self.obj.text

    def _update_text(self):
        self._set_text(self.control)

    def _set_text(self, w):
        w.SetValue("")
        w.WriteText("%s(%d points, %s)" % (self.obj.text, self.obj.size, self.obj.style))

    def _update_size(self):
        self._set_size_and_style(self.control)

    def _update_style(self):
        self._set_size_and_style(self.control)

    def _set_size_and_style(self, w):
        import wx
        if self.obj.style == 'normal':
            style, weight = wx.NORMAL, wx.NORMAL
        elif self.obj.style == 'italic':
            style, weight = wx.ITALIC, wx.NORMAL
        elif self.obj.style == 'bold':
            style, weight = wx.NORMAL, wx.BOLD
        else:
            raise NotImplementedError, 'style "%s" not supported' % self.obj.style

        f = wx.Font(self.obj.size, wx.ROMAN, style, weight, False)
        style = wx.TextAttr("BLACK",wx.NullColour,f)
        w.SetDefaultStyle(style)
        self._set_text(w)


class _PyQt4LabelEditor(Editor):
    """ _PyQt4LabelEditor is the PyQt implementation of a label editor. """

    def create_control(self, parent):

        from pyface.qt import QtCore, QtGui

        w = QtGui.QLabel(parent)
        w.setAlignment(QtCore.Qt.AlignCenter)

        self._set_text(w)
        self._set_size(w)
        self._set_style(w)

        self.obj.on_trait_change(self._update_text, 'text')
        self.obj.on_trait_change(self._update_size, 'size')
        self.obj.on_trait_change(self._update_style, 'style')

        return w

    def _name_default(self):
        return self.obj.text

    def _update_text(self):
        self._set_text(self.control)

    def _set_text(self, w):
        w.setText("%s\n(%d points, %s)" % (self.obj.text, self.obj.size, self.obj.style))

    def _update_size(self):
        self._set_size(self.control)

    def _set_size(self, w):
        f = w.font()
        f.setPointSize(self.obj.size)
        w.setFont(f)

        self._set_text(w)

    def _update_style(self):
        self._set_style(self.control)

    def _set_style(self, w):
        f = w.font()
        f.setBold(self.obj.style == 'bold')
        f.setItalic(self.obj.style == 'italic')
        w.setFont(f)

        self._set_text(w)


class ExampleEditorManager(EditorManager):
    """ The ExampleEditorManager class creates the example editors. """

    def create_editor(self, window, obj, kind):

        # Create the toolkit specific editor.
        tk_name = ETSConfig.toolkit

        if tk_name == 'wx':
            ed = _wxLabelEditor(window=window, obj=obj)
        elif tk_name == 'qt4':
            ed = _PyQt4LabelEditor(window=window, obj=obj)
        else:
            raise NotImplementedError, "unsupported toolkit: %s" % tk_name

        return ed

#### EOF ######################################################################



# Major package imports.
from setuptools import setup, find_packages

setup(
    name                 = 'PermissionsServer',
    version              = '1.0',
    author               = 'Riverbank Computing Limited',
    author_email         = 'info@riverbankcomputing.com',
    license              = 'BSD',
    zip_safe             = True,
    packages             = find_packages(),
    include_package_data = True,

    namespace_packages   = [
        'enthought'
    ],
)



"""A simple example of using the permissions framework."""


# Standard library imports.
import logging

# Enthought library imports.
from pyface.api import GUI, YES
from pyface.workbench.api import Workbench

# Local imports.
from workbench_window import ExampleWorkbenchWindow
from person import Person


# Log to stderr.
logger = logging.getLogger()
logger.addHandler(logging.StreamHandler())
logger.setLevel(logging.DEBUG)


class ExampleWorkbench(Workbench):
    """A simple example of using the workbench."""

    #### 'Workbench' interface ################################################

    # The factory (in this case simply a class) that is used to create
    # workbench windows.
    window_factory = ExampleWorkbenchWindow

    ###########################################################################
    # Private interface.
    ###########################################################################

    def _exiting_changed(self, event):
        """Called when the workbench is exiting."""

        if self.active_window.confirm('Ok to exit?') != YES:
            event.veto = True


def main(argv):
    """A simple example of using the workbench."""

    # Create the GUI.
    gui = GUI()

    # Create the workbench.
    #
    # fixme: I wouldn't really want to specify the state location here.
    # Ideally this would be part of the GUI's as DOMs idea, and the state
    # location would be an attribute picked up from the DOM hierarchy. This
    # would also be the mechanism for doing 'confirm' etc... Let the request
    # bubble up the DOM until somebody handles it.
    workbench = ExampleWorkbench(state_location=gui.state_location)

    # Create the workbench window.
    window = workbench.create_window(position=(300, 300), size=(800, 600))
    window.open()

    # This will cause a TraitsUI editor to be created.
    window.edit(Person(name='fred', age=42, salary=50000))

    # This will cause a toolkit specific editor to be created.
    window.edit("This text is implemented by a toolkit specific widget.")

    # Start the GUI event loop.
    gui.start_event_loop()


if __name__ == '__main__':
    import sys; main(sys.argv)

#### EOF ######################################################################



""" An example of using the preferences manager. """


# Enthought library imports.
from traits.api import Color, Int, Float, Str
from traitsui.api import View

# Local imports.
from apptools.preferences.api import Preferences, PreferencesHelper
from apptools.preferences.api import get_default_preferences
from apptools.preferences.api import set_default_preferences
from apptools.preferences.ui.api import PreferencesManager, PreferencesPage


# Create a preferences collection from a file and make it the default root
# preferences node for all preferences helpers etc.
set_default_preferences(Preferences(filename='example.ini'))


class AcmePreferencesPage(PreferencesPage):
    """ A preference page for the Acme preferences. """

    #### 'IPreferencesPage' interface #########################################

    # The page's category (e.g. 'General/Appearence'). The empty string means
    # that this is a top-level page.
    category = ''

    # The page's help identifier (optional). If a help Id *is* provided then
    # there will be a 'Help' button shown on the preference page.
    help_id = ''

    # The page name (this is what is shown in the preferences dialog.
    name = 'Acme'

    # The path to the preferences node that contains our preferences.
    preferences_path = 'acme'

    #### Preferences ##########################################################

    width  = Int(800)
    height = Int(600)

    #### Traits UI views ######################################################

    view = View('width', 'height')


class AcmeWorkbenchPreferencesPage(PreferencesPage):
    """ A preference page for the Acme workbench preferences. """

    #### 'IPreferencesPage' interface #########################################

    # The page's category (e.g. 'General/Appearence'). The empty string means
    # that this is a top-level page.
    category = 'Acme'

    # The page's help identifier (optional). If a help Id *is* provided then
    # there will be a 'Help' button shown on the preference page.
    help_id = ''

    # The page name (this is what is shown in the preferences dialog.
    name = 'Workbench'

    # The path to the preferences node that contains our preferences.
    preferences_path = 'acme'

    #### Preferences ##########################################################

    bgcolor = Color
    fgcolor = Color

    #### Traits UI views ######################################################

    view = View('bgcolor', 'fgcolor')


# Entry point.
if __name__ == '__main__':

    # Create a manager with some pages.
    preferences_manager = PreferencesManager(
        pages = [
            AcmePreferencesPage(), AcmeWorkbenchPreferencesPage()
        ]
    )

    # Show the UI...
    preferences_manager.configure_traits()

    # Save the preferences...
    get_default_preferences().flush()

#### EOF ######################################################################



#!/usr/bin/env python

#------------------------------------------------------------------------------
# Copyright (c) 2008, Riverbank Computing Limited
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Riverbank Computing Limited
# Description: <Enthought permissions package component>
#------------------------------------------------------------------------------


# Standard library imports.
import errno
import logging
import os
import shelve
import SimpleXMLRPCServer
import socket
import sys
import time


# Log to stderr.
logger = logging.getLogger()
logger.addHandler(logging.StreamHandler())
logger.setLevel(logging.INFO)

# The default IP address to listen on.
DEFAULT_ADDR = socket.gethostbyname(socket.gethostname())

# The default port to listen on.
DEFAULT_PORT = 3800

# The default data directory.
DEFAULT_DATADIR = os.path.expanduser('~/.ets_perms_server')

# The session timeout in seconds.
SESSION_TIMEOUT = 90 * 60


class ServerImplementation(object):
    """This is a container for all the functions implemented by the server."""

    def __init__(self, data_dir, insecure, local_user_db):
        """Initialise the object."""

        self._data_dir = data_dir
        self._insecure = insecure
        self._local_user_db = local_user_db

        # Make sure we can call _close() at any time.
        self._keys = self._roles = self._assignments = self._blobs = \
                self._users = None

        # Make sure the data directory exists.
        if not os.path.isdir(self._data_dir):
            os.mkdir(self._data_dir)

        # Load the data.
        self._keys = self._open_shelf('keys')
        self._roles = self._open_shelf('roles')
        self._assignments = self._open_shelf('assignments')
        self._blobs = self._open_shelf('blobs')

        if self._local_user_db:
            self._users = self._open_shelf('users')

        # Remove any expired session keys.
        now = time.time()
        keys = self._keys.keys()
        for k in keys:
            name, perm_ids, used_at = self._keys[k]

            if now - used_at >= SESSION_TIMEOUT:
                logger.info("Expiring session key for user %s" % name)
                del self._keys[k]

        self._sync(self._keys)

    def _close(self):
        """Close all the databases."""

        if self._keys is not None:
            self._keys.close()
            self._keys = None

        if self._roles is not None:
            self._roles.close()
            self._roles = None

        if self._assignments is not None:
            self._assignments.close()
            self._assignments = None

        if self._blobs is not None:
            self._blobs.close()
            self._blobs = None

        if self._users is not None:
            if self._local_user_db:
                self._users.close()

            self._users = None

    def _open_shelf(self, name):
        """Open a shelf."""

        logger.info("Loading %s" % name)

        fname = os.path.join(self._data_dir, name)

        try:
            return shelve.open(fname, writeback=True)
        except Exception, e:
            logger.error("Unable to open %s: %s" % (fname, e))
            sys.exit(1)

    @staticmethod
    def _sync(shelf):
        try:
            shelf.sync()
        except Exception, e:
            logger.error("Unable to sync:" % e)
            Exception("An error occurred on the permissions server.")

    def _session_data(self, key):
        """Validate the session key and return the user name and list of
        permission ids."""

        # Get the session details.
        try:
            session_name, perm_ids, used_at = self._keys[key]
        except KeyError:
            # Force the timeout test to fail.
            used_at = -SESSION_TIMEOUT

        # See if the session should be timed out.
        now = time.time()
        if now - used_at >= SESSION_TIMEOUT:
            try:
                del self._keys[key]
            except KeyError:
                pass

            self._sync(self._keys)

            raise Exception("Your session has timed out. Please login again.")

        # Update when the session was last used.
        self._keys[key] = session_name, perm_ids, now
        self._sync(self._keys)

        return session_name, perm_ids

    def _check_user(self, key, name):
        """Check that the session is current and the session user matches the
        given user and return the permission ids."""

        session_name, perm_ids = self._session_data(key)

        if session_name != name:
            raise Exception("You do not have the appropriate authority.")

        return perm_ids

    def _check_authorisation(self, key, perm_id):
        """Check the user has the given permission."""

        # Handle the easy cases first.
        if self._insecure or self.is_empty_policy():
            return

        _, perm_ids = self._session_data(key)

        if perm_id not in perm_ids:
            raise Exception("You do not have the appropriate authority.")

    def _check_policy_authorisation(self, key):
        """Check that a policy management action is authorised."""

        self._check_authorisation(key, 'ets.permissions.manage_policy')

    def _check_users_authorisation(self, key):
        """Check that a users management action is authorised."""

        self._check_authorisation(key, 'ets.permissions.manage_users')

    def capabilities(self):
        """Return a list of capabilities that the implementation supports.  The
        full list is 'user_password', 'user_add', 'user_modify', 'user_delete'.
        """

        caps = ['user_password']

        if self._local_user_db:
            caps.extend(['user_add', 'user_modify', 'user_delete'])

        return caps

    def add_user(self, name, description, password, key=None):
        """Add a new user."""

        self._check_users_authorisation(key)

        if self._local_user_db:
            if self._users.has_key(name):
                raise Exception("The user \"%s\" already exists." % name)

            self._users[name] = (description, password)
            self._sync(self._users)
        else:
            raise Exception("Adding a user isn't supported.")

        # Return a non-None value.
        return True

    def authenticate_user(self, name, password):
        """Return the tuple of the user name, description, and blob if the user
        was successfully authenticated."""

        if self._local_user_db:
            try:
                description, pword = self._users[name]
            except KeyError:
                raise Exception("The name or password is invalid.")

            if password != pword:
                raise Exception("The name or password is invalid.")
        else:
            # FIXME
            raise Exception("Authenticating a user isn't yet supported.")

        # Create the session key.  The only reason for using a human readable
        # string is to make the test client easier to use.  We only make
        # limited attempts at creating a unique key.
        for i in range(5):
            key = ''

            for ch in os.urandom(16):
                key += '%02x' % ord(ch)

            if not self._keys.has_key(key):
                break
        else:
            # Something is seriously wrong if we get here.
            msg = "Unable to create unique session key."
            logger.error(msg)
            raise Exception(msg)

        # Get the user's permissions.
        perm_ids = []

        for r in self._assignments.get(name, []):
            _, perms = self._roles[r]
            perm_ids.extend(perms)

        # Save the session data.
        self._keys[key] = name, perm_ids, time.time()

        return key, name, description, self._blobs.get(name, {})

    def matching_users(self, name, key=None):
        """Return the full name and description of all the users that match the
        given name."""

        self._check_users_authorisation(key)

        if self._local_user_db:
            # Get any user that starts with the name.
            users = [(full_name, description) for full_name, (description, _)
                            in self._users.items()
                            if full_name.startswith(name)]

            users = sorted(users)
        else:
            # FIXME
            raise Exception("Searching for users isn't yet supported.")

        return users

    def modify_user(self, name, description, password, key=None):
        """Update the description and password for the given user."""

        self._check_users_authorisation(key)

        if self._local_user_db:
            if not self._users.has_key(name):
                raise Exception("The user \"%s\" does not exist." % name)

            self._users[name] = (description, password)
            self._sync(self._users)
        else:
            raise Exception("Modifying a user isn't supported.")

        # Return a non-None value.
        return True

    def delete_user(self, name, key=None):
        """Delete a user."""

        self._check_users_authorisation(key)

        if self._local_user_db:
            try:
                del self._users[name]
            except KeyError:
                raise UserStorageError("The user \"%s\" does not exist." % name)

            self._sync(self._users)
        else:
            raise Exception("Deleting a user isn't supported.")

        # Return a non-None value.
        return True

    def unauthenticate_user(self, key=None):
        """Unauthenticate the given user (ie. identified by the session key).
        """

        if not self._local_user_db:
            # FIXME: LDAP may or may not need anything here.
            raise Exception("Unauthenticating a user isn't yet supported.")

        # Invalidate any session key:
        if key is not None:
            try:
                del self._keys[key]
            except KeyError:
                pass

            self._sync(self._keys)

        # Return a non-None value.
        return True

    def update_blob(self, name, blob, key=None):
        """Update the blob for the given user."""

        self._check_user(key, name)

        self._blobs[name] = blob
        self._sync(self._blobs)

        # Return a non-None value.
        return True

    def update_password(self, name, password, key=None):
        """Update the password for the given user."""

        self._check_user(key, name)

        if not self._local_user_db:
            try:
                description, _ = self._users[name]
            except KeyError:
                raise Exception("The user \"%s\" does not exist." % name)

            self._users[name] = (description, password)
            self._sync(self._users)
        else:
            # FIXME
            raise Exception("Updating a user password isn't yet supported.")

        # Return a non-None value.
        return True

    def add_role(self, name, description, perm_ids, key=None):
        """Add a new role."""

        self._check_policy_authorisation(key)

        if self._roles.has_key(name):
            raise Exception("The role \"%s\" already exists." % name)

        self._roles[name] = (description, perm_ids)
        self._sync(self._roles)

        # Return a non-None value.
        return True

    def all_roles(self, key=None):
        """Return a list of all roles."""

        self._check_policy_authorisation(key)

        return [(name, description)
                for name, (description, _) in self._roles.items()]

    def delete_role(self, name, key=None):
        """Delete a role."""

        self._check_policy_authorisation(key)

        if not self._roles.has_key(name):
            raise Exception("The role \"%s\" does not exist." % name)

        del self._roles[name]

        # Remove the role from any users who have it.
        for user, role_names in self._assignments.items():
            try:
                role_names.remove(name)
            except ValueError:
                continue

            self._assignments[user] = role_names

        self._sync(self._roles)
        self._sync(self._assignments)

        # Return a non-None value.
        return True

    def get_assignment(self, user_name, key=None):
        """Return the details of the assignment for the given user name."""

        self._check_policy_authorisation(key)

        try:
            role_names = self._assignments[user_name]
        except KeyError:
            return '', []

        return user_name, role_names

    def get_policy(self, name, key=None):
        """Return the details of the policy for the given user name."""

        return name, self._check_user(key, name)

    def is_empty_policy(self):
        """Return True if there is no useful data."""

        empty = (len(self._roles) == 0 or len(self._assignments) == 0)

        # Include the users as well if the database is local.
        if self._local_user_db and len(self._users) == 0:
            empty = True

        return empty

    def matching_roles(self, name, key=None):
        """Return the full name, description and permissions of all the roles
        that match the given name."""

        self._check_policy_authorisation(key)

        # Return any role that starts with the name.
        roles = [(full_name, description, perm_ids)
                for full_name, (description, perm_ids) in self._roles.items()
                        if full_name.startswith(name)]

        return sorted(roles)

    def modify_role(self, name, description, perm_ids, key=None):
        """Update an existing role."""

        self._check_policy_authorisation(key)

        if not self._roles.has_key(name):
            raise Exception("The role \"%s\" does not exist." % name)

        self._roles[name] = (description, perm_ids)
        self._sync(self._roles)

        # Return a non-None value.
        return True

    def set_assignment(self, user_name, role_names, key=None):
        """Save the roles assigned to a user."""

        self._check_policy_authorisation(key)

        if len(role_names) == 0:
            # Delete the user, but don't worry if there is no current
            # assignment.
            try:
                del self._assignments[user_name]
            except KeyError:
                pass
        else:
            self._assignments[user_name] = role_names

        self._sync(self._assignments)

        # Return a non-None value.
        return True


class RPCServer(SimpleXMLRPCServer.SimpleXMLRPCServer):
    """A thin wrapper around SimpleXMLRPCServer that handles its
    initialisation."""

    def __init__(self, addr=DEFAULT_ADDR, port=DEFAULT_PORT,
            data_dir=DEFAULT_DATADIR, insecure=False, local_user_db=False):
        """Initialise the object."""

        SimpleXMLRPCServer.SimpleXMLRPCServer.__init__(self, (addr, port))

        self._impl = ServerImplementation(data_dir, insecure, local_user_db)
        self.register_instance(self._impl)

    def server_close(self):
        """Reimplemented to tidy up the implementation."""

        SimpleXMLRPCServer.SimpleXMLRPCServer.server_close(self)

        self._impl._close()


if __name__ == '__main__':

    # Parse the command line.
    import optparse

    p = optparse.OptionParser(description="This is an XML-RPC server that "
            "provides user, role and permissions data to a user and policy "
            "manager that is part of the ETS Permissions Framework.")

    p.add_option('--data-dir', default=DEFAULT_DATADIR, dest='data_dir',
            metavar="DIR",
            help="the server's data directory [default: %s]" % DEFAULT_DATADIR)
    p.add_option('--insecure', action='store_true', default=False,
            dest='insecure',
            help="don't require a session key for data changes")
    p.add_option('--ip-address', default=DEFAULT_ADDR, dest='addr',
            help="the IP address to listen on [default: %s]" % DEFAULT_ADDR)
    p.add_option('--local-user-db', action='store_true', default=False,
            dest='local_user_db',
            help="use a local user database instead of an LDAP directory")
    p.add_option('--port', type='int', default=DEFAULT_PORT, dest='port',
            help="the TCP port to listen on [default: %d]" % DEFAULT_PORT)

    opts, args = p.parse_args()

    if args:
        p.error("unexpected additional arguments: %s" % " ".join(args))

    # We need a decent RNG for session keys.
    if not opts.insecure:
        try:
            os.urandom(1)
        except AttributeError, NotImplementedError:
            sys.stderr.write("os.urandom() isn't implemented so the --insecure flag must be used\n")
            sys.exit(1)

    # FIXME: Add LDAP support.
    if not opts.local_user_db:
        sys.stderr.write("Until LDAP support is implemented use the --local-user-db flag\n")
        sys.exit(1)

    # Create and start the server.
    server = RPCServer(addr=opts.addr, port=opts.port, data_dir=opts.data_dir,
            insecure=opts.insecure, local_user_db=opts.local_user_db)

    if opts.insecure:
        logger.warn("Server starting in insecure mode")
    else:
        logger.info("Server starting")

    try:
        try:
            server.serve_forever()
        except KeyboardInterrupt:
            pass
        else:
            raise
    finally:
        server.server_close()

    logger.info("Server terminated")



#------------------------------------------------------------------------------
# Copyright (c) 2008, Riverbank Computing Limited
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Riverbank Computing Limited
# Description: <Enthought undo package component>
#------------------------------------------------------------------------------


# Enthought library imports.
from traits.api import Enum, HasTraits, Int, Unicode


class Label(HasTraits):
    """The Label class implements the data model for a label."""

    #### 'Label' interface ####################################################

    # The name.
    name = Unicode

    # The size in points.
    size = Int(18)

    # The style.
    style = Enum('normal', 'bold', 'italic')

    ###########################################################################
    # 'Label' interface.
    ###########################################################################

    def increment_size(self, by):
        """Increment the current font size."""

        self.size += by

    def decrement_size(self, by):
        """Decrement the current font size."""

        self.size -= by



#------------------------------------------------------------------------------
# Copyright (c) 2005, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Enthought, Inc.
# Description: <Enthought naming package component>
#------------------------------------------------------------------------------
""" A naming system explorer. """


# Standard library imports.
import os, sys

# FIXME: The below won't work in an egg-based distribution.  The real question
# is why it was here in the first place.
## Put the Enthought library on the Python path.
#sys.path.append(os.path.abspath(r'..\..\..'))

# Enthought library imports.
from apptools.naming.api import Binding, Context, ContextAdapterFactory
from apptools.naming.api import PyFSContext
from apptools.naming.adapter import *
from apptools.naming.ui.explorer import Explorer
from pyface.api import GUI
from traits.api import TraitDict, TraitList
from apptools.type_manager import TypeManager
from traits.util.resource import find_resource


# Application entry point.
if __name__ == '__main__':

    # Create the GUI (this does NOT start the GUI event loop).
    gui = GUI()

    # Create a type manager to manage context adapters.
    type_manager = TypeManager()

    # Add some interesting context adapters.
    #
    # Trait dictionaries.
    type_manager.register_type_adapters(
        ContextAdapterFactory(
            adaptee_class=TraitDict, adapter_class=TraitDictContextAdapter,
        ),

        TraitDict
    )

    # Trait lists.
    type_manager.register_type_adapters(
        ContextAdapterFactory(
            adaptee_class=TraitList, adapter_class=TraitListContextAdapter,
        ),

        TraitList
    )

    # Python dictionaries.
    type_manager.register_type_adapters(
        ContextAdapterFactory(
            adaptee_class=dict, adapter_class=DictContextAdapter,
        ),

        dict
    )

    # Python lists.
    type_manager.register_type_adapters(
        ContextAdapterFactory(
            adaptee_class=list, adapter_class=ListContextAdapter,
        ),

        list
    )

    # Python objects.
    type_manager.register_type_adapters(
        InstanceContextAdapterFactory(), object
    )

    # Get the path to the data directory
    data_path = os.path.join('examples','naming','data')
    full_path = find_resource('AppTools', data_path, alt_path='data',
        return_path=True)

    # Create the root context.
    root = PyFSContext(path=full_path)
    root.environment[Context.TYPE_MANAGER] = type_manager

    # Create and open the main window.
    window = Explorer(root=Binding(name='Root', obj=root))
    window.open()

    # Start the GUI event loop.
    gui.start_event_loop()

##### EOF #####################################################################



"""An editor implemented by a toolkit specific widget.  This demonstrates the
ability to apply permissions to toolkit specific widgets.
"""


# Enthought library imports.
from traits.etsconfig.api import ETSConfig
from apptools.permissions.api import SecureProxy
from pyface.workbench.api import Editor

# Local imports.
from permissions import EnableWidgetPerm


class ToolkitEditor(Editor):
    """A workbench editor that displays string with a permission attached to
    the toolkit widget.
    """

    def create_control(self, parent):
        """Create the toolkit specific control."""

        tk = ETSConfig.toolkit

        if tk == 'wx':
            import wx

            control = wx.StaticText(parent, -1, self.obj,
                    style=wx.ALIGN_CENTER)
        elif tk == 'qt4':
            from PyQt4 import QtCore, QtGui

            control = QtGui.QLabel(self.obj, parent)
            control.setAlignment(QtCore.Qt.AlignHCenter)
        else:
            raise ValueError, "unsupported toolkit: %s" % tk

        # Return the wrapped control.
        return SecureProxy(control, [EnableWidgetPerm])

    def destroy_control(self):
        """Create the toolkit specific control."""

        tk = ETSConfig.toolkit

        if tk == 'wx':
            self.control.Destroy()
        elif tk == 'qt4':
            self.control.setParent(None)

    def _name_default(self):
        """Show the toolkit in the editor name."""

        return str(self)

    ###########################################################################
    # 'object' interface.
    ###########################################################################

    def __str__(self):
        """Return an informal string representation of the object."""

        return ETSConfig.toolkit + " Editor"

#### EOF ######################################################################



class Foo0:
    """ The original class written with no expectation of being upgraded """
    def __init__(self):
        self.prenom = 'didier'
        self.surnom = 'enfant'


class Foo1:
    """ Now to handle both Foo v0 and Foo v1 we need to add more code ..."""

    def __init__(self, firstname, lastname):
        """ This does not get called when the class is unpickled."""
        self.firstname = firstname
        self.lastname = lastname


class Foo:

    def __str__(self):
        result = ['-----------------------------------------------------------']
        keys = dir(self)
        for key in keys:
             result.append('%s ---> %s' % (key, getattr(self, key)))
        result.append('-----------------------------------------------------------')
        return '\n'.join(result)

    def __setstate__(self, state):
        print 'calling setstate on the real Foo'
        state['set'] = True
        self.__dict__.update(state)



def save(fname, str):
    f=open(fname, 'w')
    f.write(str)
    f.close()
    return


if __name__ == '__main__':

    # Create dummy test data .......
    #from cStringIO import StringIO
    import pickle

    obj = Foo0()
    print obj
    t0 = pickle.dumps(obj) #.replace('Foo0', 'Foo')
    save('foo0.txt', t0)

    '''obj = Foo1('duncan', 'child')
    t1 = pickle.dumps(obj).replace('Foo1', 'Foo')
    save('foo1.txt', t1)

    obj = Foo2('duncan child')
    t2 = pickle.dumps(obj).replace('Foo2', 'Foo')
    save('foo2.txt', t2)

    obj = Foo3('duncan child')
    t3 = pickle.dumps(obj).replace('Foo3', 'Foo')
    save('foo3.txt', t3)
    '''

    print '===================================================================='

    from apptools.persistence.versioned_unpickler import VersionedUnpickler
    from update1 import Update1
    # Try and read them back in ...
    f = open('foo0.txt')

    import sys
    rev = 1
    __import__('integrationtests.persistence.update%d' % rev)
    mod = sys.modules['integrationtests.persistence.update%d' % rev]
    klass = getattr(mod, 'Update%d' % rev)
    updater = klass()
    print '%s %s' % (rev, updater)

    p = VersionedUnpickler(f, updater).load()
    print p
    print 'Restored version %s %s' % (p.lastname, p.firstname)
    #print p.set




from email.mime.multipart import MIMEMultipart
import io
import os
import shutil
import tempfile
import unittest

try:
    # On Python 3, mock is part of the standard library,
    from unittest import mock
except ImportError:
    # Whereas on Python 2 it is not.
    import mock

from apptools.logger.agent.attachments import Attachments


class AttachmentsTestCase(unittest.TestCase):
    def setUp(self):
        self.tmpdir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, self.tmpdir)

        self.tmpfile = os.path.join(self.tmpdir, "dummy_file.txt")
        with io.open(self.tmpfile, 'w', encoding='utf8') as filehandle:
            filehandle.write(u"Dummy data in dummy file for dummies")

    def test_attaching_workspace(self):
        class DummyWorkspace(object):
            path = self.tmpdir

        class MockedApplication(object):
            tmpdir = self.tmpdir
            def get_service(self, service_id):
                return DummyWorkspace()

        attachments = Attachments(
            application=MockedApplication(),
            message=MIMEMultipart()
        )
        attachments.package_workspace()

        message = attachments.message
        self.assertTrue(message.is_multipart())
        payload = message.get_payload()
        self.assertEqual(len(payload), 1)

    def test_attaching_single_project(self):
        class DummySingleProject(object):
            location = self.tmpdir

        class MockedApplication(object):
            tmpdir = self.tmpdir
            def get_service(self, service_id):
                return DummySingleProject()

        attachments = Attachments(
            application=MockedApplication(),
            message=MIMEMultipart()
        )
        attachments.package_single_project()

        message = attachments.message
        self.assertTrue(message.is_multipart())
        payload = message.get_payload()
        self.assertEqual(len(payload), 1)



"""The definitions of all the permissions used in the example."""


from apptools.permissions.api import Permission


# Access to the Debug view.
DebugViewPerm = Permission(id='ets.permissions.example.debug.view',
        description=u"Use the debug view")

# Add a new person.
NewPersonPerm = Permission(id='ets.permissions.example.person.new',
        description=u"Add a new person")

# Update a person's age.
UpdatePersonAgePerm = Permission(id='ets.permissions.example.person.age.update',
        description=u"Update a person's age")

# View or update a person's salary.
PersonSalaryPerm = Permission(id='ets.permissions.example.person.salary',
        description=u"View or update a person's salary")

# Enable the example toolkit specific widget.
EnableWidgetPerm = Permission(id='ets.permissions.example.widget',
        description=u"Enable the example toolkit specific widget")



#------------------------------------------------------------------------------
# Copyright (c) 2007, Riverbank Computing Limited
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Riverbank Computing Limited
# Description: <Enthought undo package component>
#------------------------------------------------------------------------------


# Enthought library imports.
from traits.api import Instance, Int, Unicode
from apptools.undo.api import AbstractCommand

# Local imports.
from model import Label


class LabelIncrementSizeCommand(AbstractCommand):
    """ The LabelIncrementSizeCommand class is a command that increases the
    size of a label's text.  This command will merge multiple increments
    togther.
    """

    #### 'ICommand' interface #################################################

    # The data being operated on.
    data = Instance(Label)

    # The name of the command.
    name = Unicode("&Increment size")

    #### Private interface ####################################################

    _incremented_by = Int

    ###########################################################################
    # 'ICommand' interface.
    ###########################################################################

    def do(self):
        self.data.increment_size(1)
        self._incremented_by = 1

    def merge(self, other):
        # We can merge if the other command is the same type (or a sub-type).
        if isinstance(other, type(self)):
            self._incremented_by += 1
            merged = True
        else:
            merged = False

        return merged

    def redo(self):
        self.data.increment_size(self._incremented_by)

    def undo(self):
        self.data.decrement_size(self._incremented_by)


class LabelDecrementSizeCommand(AbstractCommand):
    """ The LabelDecrementSizeCommand class is a command that decreases the
    size of a label's text.  This command will merge multiple decrements
    togther.
    """

    #### 'ICommand' interface #################################################

    # The data being operated on.
    data = Instance(Label)

    # The name of the command.
    name = Unicode("&Decrement size")

    #### Private interface ####################################################

    _decremented_by = Int

    ###########################################################################
    # 'ICommand' interface.
    ###########################################################################

    def do(self):
        self.data.decrement_size(1)
        self._decremented_by = 1

    def merge(self, other):
        # We can merge if the other command is the same type (or a sub-type).
        if isinstance(other, type(self)):
            self._decremented_by += 1
            merged = True
        else:
            merged = False

        return merged

    def redo(self):
        self.data.decrement_size(self._decremented_by)

    def undo(self):
        self.data.increment_size(self._decremented_by)


class LabelNormalFontCommand(AbstractCommand):
    """ The LabelNormalFontCommand class is a command that sets a normal font
    for a label's text.
    """

    #### 'ICommand' interface #################################################

    # The data being operated on.
    data = Instance(Label)

    # The name of the command.
    name = Unicode("&Normal font")

    ###########################################################################
    # 'ICommand' interface.
    ###########################################################################

    def do(self):
        # Save the old value.
        self._saved = self.data.style

        # Calling redo() is a convenient way to update the model now that the
        # old value is saved.
        self.redo()

    def redo(self):
        self.data.style = 'normal'

    def undo(self):
        self.data.style = self._saved


class LabelBoldFontCommand(AbstractCommand):
    """ The LabelNormalFontCommand class is a command that sets a bold font for
    a label's text.
    """

    #### 'ICommand' interface #############################################

    # The data being operated on.
    data = Instance(Label)

    # The name of the command.
    name = Unicode("&Bold font")

    ###########################################################################
    # 'ICommand' interface.
    ###########################################################################

    def do(self):
        # Save the old value.
        self._saved = self.data.style

        # Calling redo() is a convenient way to update the model now that the
        # old value is saved.
        self.redo()

    def redo(self):
        self.data.style = 'bold'

    def undo(self):
        self.data.style = self._saved


class LabelItalicFontCommand(AbstractCommand):
    """ The LabelNormalFontCommand class is a command that sets an italic font
    for a label's text.
    """

    #### 'ICommand' interface #################################################

    # The data being operated on.
    data = Instance(Label)

    # The name of the command.
    name = Unicode("&Italic font")

    ###########################################################################
    # 'ICommand' interface.
    ###########################################################################

    def do(self):
        # Save the old value.
        self._saved = self.data.style

        # Calling redo() is a convenient way to update the model now that the
        # old value is saved.
        self.redo()

    def redo(self):
        self.data.style = 'italic'

    def undo(self):
        self.data.style = self._saved



#------------------------------------------------------------------------------
# Copyright (c) 2008, Riverbank Computing Limited
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Riverbank Computing Limited
# Description: <Enthought undo package component>
#------------------------------------------------------------------------------


# Standard library imports.
import logging

# Enthought library imports.
from pyface.api import GUI, YES
from pyface.workbench.api import Workbench

# Local imports.
from example_undo_window import ExampleUndoWindow
from model import Label


# Log to stderr.
logging.getLogger().addHandler(logging.StreamHandler())
logging.getLogger().setLevel(logging.DEBUG)


class ExampleUndo(Workbench):
    """ The ExampleUndo class is a workbench that creates ExampleUndoWindow
    windows.
    """

    #### 'Workbench' interface ################################################

    # The factory (in this case simply a class) that is used to create
    # workbench windows.
    window_factory = ExampleUndoWindow

    ###########################################################################
    # Private interface.
    ###########################################################################

    def _exiting_changed(self, event):
        """ Called when the workbench is exiting. """

        if self.active_window.confirm('Ok to exit?') != YES:
            event.veto = True

        return


def main(argv):
    """ A simple example of using the the undo framework in a workbench. """

    # Create the GUI.
    gui = GUI()

    # Create the workbench.
    workbench = ExampleUndo(state_location=gui.state_location)

    window = workbench.create_window(position=(300, 300), size=(400, 300))
    window.open()

    # Create some objects to edit.
    label = Label(text="Label")
    label2 = Label(text="Label2")

    # Edit the objects.
    window.edit(label)
    window.edit(label2)

    # Start the GUI event loop.
    gui.start_event_loop()

    return


if __name__ == '__main__':
    import sys; main(sys.argv)

#### EOF ######################################################################



#------------------------------------------------------------------------------
# Copyright (c) 2008, Riverbank Computing Limited
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Riverbank Computing Limited
# Description: <Enthought application scripting package component>
#------------------------------------------------------------------------------


# Enthought library imports.
from traits.api import Any, Instance, Str, Unicode
from pyface.action.api import Action
from pyface.workbench.api import WorkbenchWindow

# Local imports.
from model import Label


class BoundAction(Action):
    """An action with a bound object.  The action is automatically disabled if
    the bound object is None."""

    #### 'BoundAction' interface ##############################################

    # The bound object.
    obj = Any

    # The optional trait on obj that we are synch'ed with.
    trait_name = Str

    ###########################################################################
    # 'object' interface.
    ###########################################################################

    def __init__(self, **traits):
        """Initialise the object."""

        super(BoundAction, self).__init__(**traits)

        # Fake an obj change to set the initial state.
        self._obj_changed(None, self.obj)

    ###########################################################################
    # Traits handlers.
    ###########################################################################

    def _obj_changed(self, old, new):
        """Invoked when the bound object changes."""

        if old is not None:
            if self.trait_name:
                # Ignore any changes to the old object.
                old.on_trait_change(self._trait_changed, self.trait_name,
                        remove=True)

        enabled = False

        if new is not None:
            if self.trait_name:
                # Check for any changes on the new object.
                new.on_trait_change(self._trait_changed, self.trait_name)

                # Get the current state.
                if getattr(new, self.trait_name):
                    enabled = True
            else:
                enabled = True

        self.enabled = enabled

    def _trait_changed(self, new):
        """Invoked when the trait on the bound object changes."""

        self.enabled = new


class BoundWorkbenchAction(BoundAction):
    """A bound action whose object is being edited in a workbench editor.  The
    action is automatically rebound when the active editor changes."""

    #### 'BoundWorkbenchAction' interface #####################################

    # The type of the object that we will be enabled for.  If it is None then
    # we will be enabled for all types.
    trait_type = Any

    # The workbench window containing the action.
    window = Instance(WorkbenchWindow)

    ###########################################################################
    # 'object' interface.
    ###########################################################################

    def __init__(self, **traits):
        """Initialise the object."""

        super(BoundWorkbenchAction, self).__init__(**traits)

        self.window.on_trait_change(self._editor_changed, 'active_editor')

        # Fake an editor change to set the initial state.
        self._editor_changed(self.window.active_editor)

    ###########################################################################
    # Traits handlers.
    ###########################################################################

    def _editor_changed(self, new):
        """Invoked when the active editor changes."""

        obj = None

        if new is not None:
            if self.trait_type is None:
                obj = new.obj
            elif isinstance(new.obj, self.trait_type):
                obj = new.obj

        self.obj = obj


class LabelAction(BoundWorkbenchAction):
    """ The LabelAction class is the base class for all actions that operate on
    a Label.
    """

    #### 'BoundWorkbenchAction' interface #####################################

    # The type of the object that we will be enabled for.
    trait_type = Label

    #### 'BoundAction' interface ##############################################

    # The bound object.
    obj = Instance(Label)


class LabelIncrementSizeAction(LabelAction):
    """ The LabelIncrementSizeAction class is a action that increases the size
    of a label's text.
    """

    #### 'Action' interface ###################################################

    # The name of the action.
    name = Unicode("&Increment size")

    ###########################################################################
    # 'Action' interface.
    ###########################################################################

    def perform(self, event):
        self.obj.increment_size(1)


class LabelDecrementSizeAction(LabelAction):
    """ The LabelDecrementSizeAction class is a action that decreases the size
    of a label's text.
    """

    #### 'Action' interface ###################################################

    # The name of the action.
    name = Unicode("&Decrement size")

    ###########################################################################
    # 'Action' interface.
    ###########################################################################

    def perform(self, event):
        self.obj.decrement_size(1)


class LabelNormalFontAction(LabelAction):
    """ The LabelNormalFontAction class is a action that sets a normal font for
    a label's text.
    """

    #### 'Action' interface ###################################################

    # The name of the action.
    name = Unicode("&Normal font")

    ###########################################################################
    # 'Action' interface.
    ###########################################################################

    def perform(self, event):
        self.obj.style = 'normal'


class LabelBoldFontAction(LabelAction):
    """ The LabelNormalFontAction class is a action that sets a bold font for a
    label's text.
    """

    #### 'Action' interface ###################################################

    # The name of the action.
    name = Unicode("&Bold font")

    ###########################################################################
    # 'Action' interface.
    ###########################################################################

    def perform(self, event):
        self.obj.style = 'bold'


class LabelItalicFontAction(LabelAction):
    """ The LabelNormalFontAction class is a action that sets an italic font
    for a label's text.
    """

    #### 'Action' interface ###################################################

    # The name of the action.
    name = Unicode("&Italic font")

    ###########################################################################
    # 'Action' interface.
    ###########################################################################

    def perform(self, event):
        self.obj.style = 'italic'



# Update class names from the immediately prior version only
# to ensure that cycles are not possible

from apptools.persistence.updater import Updater

def cleanup_foo(self, state):

    print 'cleaning up Foo0'
    state['firstname'] = state['prenom']
    state['lastname'] = state['surnom']

    del state['prenom']
    del state['surnom']

    '''for key in state:
        print '%s state ---> %s' % (key, state[key])
    '''

    #self.__setstate_original__(state)
    self.__dict__.update(state)


def update_project(self, state):
    print 'updating to v1'
    metadata = state['metadata']
    metadata['version'] = 1
    metadata['diesel'] = 'E300TD'
    return state


class Update1(Updater):

    def __init__(self):

        self.refactorings = {
            ("__main__", "Foo0"): ("__main__", "Foo"),
        }

        self.setstates = {
            ("cplab.project", "Project"):  update_project
        }



#------------------------------------------------------------------------------
# Copyright (c) 2008, Riverbank Computing Limited
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Riverbank Computing Limited
# Description: <Enthought undo package component>
#------------------------------------------------------------------------------


# Enthought library imports.
from pyface.action.api import Action, Group, MenuManager
from pyface.workbench.api import WorkbenchWindow
from pyface.workbench.action.api import MenuBarManager, ToolBarManager
from traits.api import Instance
from apptools.undo.action.api import CommandAction, RedoAction, UndoAction

# Local imports.
from example_editor_manager import ExampleEditorManager
from commands import LabelIncrementSizeCommand, LabelDecrementSizeCommand, \
        LabelNormalFontCommand, LabelBoldFontCommand, LabelItalicFontCommand


class ExampleUndoWindow(WorkbenchWindow):
    """ The ExampleUndoWindow class is a workbench window that contains example
    editors that demonstrate the use of the undo framework.
    """

    #### Private interface ####################################################

    # The action that exits the application.
    _exit_action = Instance(Action)

    # The File menu.
    _file_menu = Instance(MenuManager)

    # The Label menu.
    _label_menu = Instance(MenuManager)

    # The Undo menu.
    _undo_menu = Instance(MenuManager)

    ###########################################################################
    # Private interface.
    ###########################################################################

    #### Trait initialisers ###################################################

    def __file_menu_default(self):
        """ Trait initialiser. """

        return MenuManager(self._exit_action, name="&File")

    def __undo_menu_default(self):
        """ Trait initialiser. """

        undo_manager = self.workbench.undo_manager

        undo_action = UndoAction(undo_manager=undo_manager)
        redo_action = RedoAction(undo_manager=undo_manager)

        return MenuManager(undo_action, redo_action, name="&Undo")

    def __label_menu_default(self):
        """ Trait initialiser. """

        size_group = Group(CommandAction(command=LabelIncrementSizeCommand),
                           CommandAction(command=LabelDecrementSizeCommand))

        normal = CommandAction(id='normal', command=LabelNormalFontCommand,
                               style='radio', checked=True)
        bold = CommandAction(id='bold', command=LabelBoldFontCommand,
                             style='radio')
        italic = CommandAction(id='italic', command=LabelItalicFontCommand,
                               style='radio')

        style_group = Group(normal, bold, italic, id='style')

        return MenuManager(size_group, style_group, name="&Label")

    def __exit_action_default(self):
        """ Trait initialiser. """

        return Action(name="E&xit", on_perform=self.workbench.exit)

    def _editor_manager_default(self):
        """ Trait initialiser. """

        return ExampleEditorManager()

    def _menu_bar_manager_default(self):
        """ Trait initialiser. """

        return MenuBarManager(self._file_menu, self._label_menu,
                self._undo_menu, window=self)

    def _tool_bar_manager_default(self):
        """ Trait initialiser. """

        return ToolBarManager(self._exit_action, show_tool_names=False)

    def _active_editor_changed(self, old, new):
        """ Trait handler. """

        # Tell the undo manager about the new command stack.
        if old is not None:
            old.command_stack.undo_manager.active_stack = None

        if new is not None:
            new.command_stack.undo_manager.active_stack = new.command_stack


        # Walk the label editor menu.
        for grp in self._label_menu.groups:
            for itm in grp.items:
                action = itm.action

                # Enable the action and set the command stack and data if there
                # is a new editor.
                if new is not None:
                    action.enabled = True
                    action.command_stack = new.command_stack
                    action.data = new.obj

                    # FIXME v3: We should just be able to check the menu option
                    # corresponding to the style trait - but that doesn't seem
                    # to uncheck the other options in the group.  Even then the
                    # first switch to another editor doesn't update the menus
                    # (though subsequent ones do).
                    if grp.id == 'style':
                        action.checked = (action.data.style == action.id)
                else:
                    action.enabled = False

#### EOF ######################################################################



#------------------------------------------------------------------------------
# Copyright (c) 2008, Riverbank Computing Limited
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Riverbank Computing Limited
# Description: <Enthought permissions package component>
#------------------------------------------------------------------------------


# Standard library imports.
import cPickle as pickle
import errno
import os
import socket
import xmlrpclib

# Enthought library imports.
from traits.etsconfig.api import ETSConfig
from pyface.api import confirm, YES


# The default IP address to connect to.
DEFAULT_ADDR = socket.gethostbyname(socket.gethostname())

# The default port to connect to.
DEFAULT_PORT = 3800


class ProxyServer(xmlrpclib.ServerProxy):
    """This is a thin wrapper around xmlrpclib.ServerProxy that handles the
    server address and error reporting."""

    # The name of the user data cache file.
    _cache_file = os.path.join(ETSConfig.application_home,
            'ets_perms_user_cache')

    def __init__(self):
        """Initialise the object.  The server address and TCP/IP port are taken
        from the ETS_PERMS_SERVER environment variable if set."""

        self._server = os.environ.get('ETS_PERMS_SERVER',
                '%s:%d' % (DEFAULT_ADDR, DEFAULT_PORT))

        xmlrpclib.ServerProxy.__init__(self, uri='http://%s' % self._server)

        # The session key.  It is an empty string if the user is not
        # authenticated and None if the user is in "disconnected" mode.
        self.key = ''

        # The user data cache.  This is a tuple of the user description, blob
        # and list of permission ids when the session key is not an empty
        # string.
        self.cache = None

    def write_cache(self):
        """Write the user cache to persistent storage."""

        f = open(self._cache_file, 'w')
        pickle.dump(self.cache, f)
        f.close()

    def read_cache(self):
        """Read the user cache from persistent storage.  Returns False if
        there was no cache to read."""

        try:
            f = open(self._cache_file, 'r')

            try:
                if confirm(None, "It was not possible to connect to the "
                        "permissions server. Instead you can use the settings "
                        "used when you last logged in from this system. If "
                        "you do this then any changes made to settings "
                        "normally held on the permissions server will be lost "
                        "when you next login successfully.\n\nDo you want to "
                        "use the saved settings?") != YES:
                    raise Exception("")

                try:
                    self.cache = pickle.load(f)
                except:
                    raise Exception("Unable to read %s." % self._cache_file)
            finally:
                f.close()
        except IOError, e:
            if e.errno == errno.ENOENT:
                # There is no cache file.
                return False

            raise Exception("Unable to open %s: %s." % (self._cache_file, e))

        return True

    def error(self, e):
        """Return a user friendly string describing the given exception."""

        if isinstance(e, socket.error):
            err, msg = e.args

            if err == errno.ECONNREFUSED:
                emsg = "Unable to connect to permissions server at %s." % self._server
            else:
                emsg = "Socket error to permissions server at %s." % self._server
        elif isinstance(e, xmlrpclib.Fault):
            # Extract the text of the exception.  If we don't recognise the
            # format then display the lot.
            tail = e.faultString.find(':')
            if tail < 0:
                emsg = "Unexpected exception from permissions server at %s: %s\n" % (self._server, e.faultString)
            else:
                emsg = e.faultString[tail + 1:]
        else:
            # Let any existing error handling deal with it.
            raise e

        return emsg


# Create a singleton.
ProxyServer = ProxyServer()



#------------------------------------------------------------------------------
# Copyright (c) 2008, Riverbank Computing Limited
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Riverbank Computing Limited
# Description: <Enthought application scripting package component>
#------------------------------------------------------------------------------


# Enthought library imports.
from pyface.action.api import Action, Group, MenuManager
from pyface.workbench.api import WorkbenchWindow
from pyface.workbench.action.api import MenuBarManager, \
        ToolBarManager
from traits.api import Instance, on_trait_change
from apptools.appscripting.api import get_script_manager
from apptools.appscripting.action.api import StartRecordingAction, \
        StopRecordingAction

# Local imports.
from example_editor_manager import ExampleEditorManager
from actions import LabelIncrementSizeAction, LabelDecrementSizeAction, \
        LabelNormalFontAction, LabelBoldFontAction, LabelItalicFontAction


class ExampleScriptWindow(WorkbenchWindow):
    """ The ExampleScriptWindow class is a workbench window that contains
    example editors that demonstrate the use of the application scripting
    framework.
    """

    #### Private interface ####################################################

    # The action that exits the application.
    _exit_action = Instance(Action)

    # The File menu.
    _file_menu = Instance(MenuManager)

    # The Label menu.
    _label_menu = Instance(MenuManager)

    # The Scripts menu.
    _scripts_menu = Instance(MenuManager)

    ###########################################################################
    # Private interface.
    ###########################################################################

    #### Trait initialisers ###################################################

    def __file_menu_default(self):
        """ Trait initialiser. """

        return MenuManager(self._exit_action, name="&File")

    def __label_menu_default(self):
        """ Trait initialiser. """

        size_group = Group(LabelIncrementSizeAction(window=self),
                LabelDecrementSizeAction(window=self))

        normal = LabelNormalFontAction(window=self, id='normal', style='radio',
                checked=True)
        bold = LabelBoldFontAction(window=self, id='bold', style='radio')
        italic = LabelItalicFontAction(window=self, id='italic', style='radio')

        style_group = Group(normal, bold, italic, id='style')

        return MenuManager(size_group, style_group, name="&Label")

    def __scripts_menu_default(self):
        """ Trait initialiser. """

        # ZZZ: This is temporary until we put the script into a view.
        get_script_manager().on_trait_event(self._on_script_updated,
                'script_updated')

        return MenuManager(StartRecordingAction(), StopRecordingAction(),
                name="&Scripts")

    def __exit_action_default(self):
        """ Trait initialiser. """

        return Action(name="E&xit", on_perform=self.workbench.exit)

    def _editor_manager_default(self):
        """ Trait initialiser. """

        return ExampleEditorManager()

    def _menu_bar_manager_default(self):
        """ Trait initialiser. """

        return MenuBarManager(self._file_menu, self._label_menu,
                self._scripts_menu, window=self)

    def _tool_bar_manager_default(self):
        """ Trait initialiser. """

        return ToolBarManager(self._exit_action, show_tool_names=False)

    # ZZZ: This is temporary until we put the script into a view.
    def _on_script_updated(self, script_manager):
        script = script_manager.script

        if script:
            print script,
        else:
            print "Script empty"

#### EOF ######################################################################



"""A simple example of an object model."""


# Enthought library imports.
from apptools.permissions.api import SecureHandler
from traits.api import HasTraits, Int, Unicode
from traitsui.api import Item, View

# Local imports.
from permissions import UpdatePersonAgePerm, PersonSalaryPerm


class Person(HasTraits):
    """A simple example of an object model"""

    # Name.
    name = Unicode

    # Age in years.
    age = Int

    # Salary.
    salary = Int

    # Define the default view with permissions attached.
    age_perm = UpdatePersonAgePerm
    salary_perm = PersonSalaryPerm

    traits_view = View(
            Item(name='name'),
            Item(name='age', enabled_when='object.age_perm.granted'),
            Item(name='salary', visible_when='object.salary_perm.granted'),
            handler=SecureHandler)

    ###########################################################################
    # 'object' interface.
    ###########################################################################

    def __str__(self):
        """Return an informal string representation of the object."""

        return self.name

#### EOF ######################################################################



#------------------------------------------------------------------------------
# Copyright (c) 2005, Enthought, Inc.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Enthought, Inc.
# Description: <Enthought naming package component>
#------------------------------------------------------------------------------
""" A simple naming example. """


# Standard library imports.
import os, sys

# Enthought library imports.
from apptools.naming.api import Context, InitialContext


# Application entry point.
if __name__ == '__main__':

    # Set up the naming environment.
    klass_name = "apptools.naming.InitialContextFactory"
    klass_name = "apptools.naming.PyFSInitialContextFactory"
    environment = {Context.INITIAL_CONTEXT_FACTORY : klass_name}

    # Create an initial context.
    context = InitialContext(environment)
    context.path = os.getcwd()
    print 'Context', context, context.path
    print 'Names', context.list_names('')

##### EOF #####################################################################



"""The secured DebugView for the permissions framework example."""


# Enthought library imports.
from pyface.workbench.debug.api import DebugView
from apptools.permissions.api import SecureProxy

# Local imports.
from permissions import DebugViewPerm


class SecuredDebugView(DebugView):
    """A secured DebugView."""

    ###########################################################################
    # 'IWorkbenchPart' interface.
    ###########################################################################

    def create_control(self, parent):
        """Reimplemented to secure the created control."""

        control = DebugView.create_control(self, parent)

        return SecureProxy(control, permissions=[DebugViewPerm])

#### EOF ######################################################################



#------------------------------------------------------------------------------
# Copyright (c) 2008, Riverbank Computing Limited
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in enthought/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
# Thanks for using Enthought open source!
#
# Author: Riverbank Computing Limited
# Description: <Enthought application scripting package component>
#------------------------------------------------------------------------------


# Enthought library imports.
from traits.api import Enum, HasTraits, Int, Unicode
from apptools.appscripting.api import scriptable, Scriptable


class Label(HasTraits):
    """ The Label class implements the data model for a label. """

    #### 'Label' interface ####################################################

    # The name.
    name = Unicode

    # The size in points.
    size = Int(18)

    # The style.
    style = Scriptable(Enum('normal', 'bold', 'italic'))

    ###########################################################################
    # 'Label' interface.
    ###########################################################################

    @scriptable
    def __init__(self, **traits):
        """ Initialise the object.  We only implement this so that it can be
        decorated and so the script manager knows how to recreate it. """

        super(Label, self).__init__(**traits)

    @scriptable
    def increment_size(self, by):
        """ Increment the current font size.  This demonstrates a scriptable
        method.
        """

        self.size += by

    @scriptable
    def decrement_size(self, by):
        """ decrement the current font size.  This demonstrates a scriptable
        method.
        """

        self.size -= by
